﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Device.Location;
using System.Diagnostics;
using System.IO;
using System.IO.IsolatedStorage;
using System.Linq;
using System.Net;
using System.Xml.Linq;
using Microsoft.Phone.Shell;
using VeloToulouse.Code;
using VeloToulouse.Entites;

namespace VeloToulouse
{
    public class StationsService
    {
        private const string StationUrl = "http://www.velo.toulouse.fr/service/carto";
        private static StationsService _current;
        private GeoCoordinateWatcher watcher;

        private StationsService()
        {
            Stations = new ObservableCollection<Station>();
            StationsFavorites = new ObservableCollection<Station>();
            StationsNearMe = new ObservableCollection<Station>();
            PhoneApplicationService.Current.Deactivated += Current_Deactivated;
        }

        public ObservableCollection<Station> Stations { get; set; }
        public ObservableCollection<Station> StationsFavorites { get; set; }
        public ObservableCollection<Station> StationsNearMe { get; set; }

        public static StationsService Current
        {
            get
            {
                if (_current == null)
                    _current = new StationsService();
                return _current;
            }
            private set { _current = value; }
        }

        public void WakeUpService()
        {
            Stations = (ObservableCollection<Station>) PhoneApplicationService.Current.State["Stations"];
            StationsFavorites =
                (ObservableCollection<Station>) PhoneApplicationService.Current.State["StationsFavorites"];
        }

        private void Current_Deactivated(object sender, DeactivatedEventArgs e)
        {
            PhoneApplicationService.Current.State["Stations"] = Stations;
            PhoneApplicationService.Current.State["StationsFavorites"] = StationsFavorites;
        }

        public Station GetStationById(string id)
        {
            return (from s in Stations
                    where s.Id.Equals(id)
                    select s).FirstOrDefault();
        }

        public void GetListeStationsAsync()
        {
            var webClient = new WebClient();

            webClient.OpenReadCompleted += LoadCompleted;
            webClient.OpenReadAsync(new Uri(StationUrl));
        }

        private void LoadCompleted(object sender, OpenReadCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                InvokeGetListeStationsCompleted(null);
                return;
            }

            Stream s = e.Result;
            XDocument xdoc = XDocument.Load(s);

            List<Station> res = (xdoc.Descendants("marker").Select(
                i => new Station
                         {
                             Id = i.Attribute("number").Value,
                             Nom = i.Attribute("name").Value,
                             Adresse = i.Attribute("fullAddress").Value,
                             Latitude = i.Attribute("lat").Value,
                             Longitude = i.Attribute("lng").Value
                         })).ToList();

            res.ForEach(item => Stations.Add(item));
            LoadFavoris();
            InvokeGetListeStationsCompleted(new EventArgs());
#if DEBUG
            StationsNearMe.Add(Stations.LastOrDefault());
#else

            watcher = new GeoCoordinateWatcher(GeoPositionAccuracy.Default); 
            watcher.MovementThreshold = 20; // permet d'ignorer le bruit
            watcher.PositionChanged += watcher_PositionChanged;
            watcher.Start();
#endif
        }

        private void watcher_PositionChanged(object sender, GeoPositionChangedEventArgs<GeoCoordinate> e)
        {
            GeoPosition<GeoCoordinate> position = watcher.Position;

            foreach (Station station in Stations)
            {
                if (
                    CalculateDistanceBetweenGPSCoordinates(Convert.ToDouble(station.Longitude),
                                                           Convert.ToDouble(station.Latitude),
                                                           position.Location.Longitude, position.Location.Latitude) <= 2)
                    StationsNearMe.Add(station);
            }
            
        }

        public static double CalculateDistanceBetweenGPSCoordinates(double lon1, double lat1, double lon2, double lat2)
        {
            const double R = 3956;
            const double degreesToRadians = Math.PI/180;

            //convert from fractional degrees (GPS) to radians 
            lon1 *= degreesToRadians;
            lat1 *= degreesToRadians;
            lon2 *= degreesToRadians;
            lat2 *= degreesToRadians;

            double dlon = lon2 - lon1;
            double dlat = lat2 - lat1;
            double a = Math.Pow(Math.Sin(dlat/2), 2) + Math.Cos(lat1)*Math.Cos(lat2)*Math.Pow(Math.Sin(dlon/2), 2);
            double c = 2*Math.Atan2(Math.Sqrt(a), Math.Sqrt(1 - a));
            double d = R*c;

            return d;
        }

        public event EventHandler GetListeStationsCompleted;

        public void InvokeGetListeStationsCompleted(EventArgs e)
        {
            EventHandler handler = GetListeStationsCompleted;
            if (handler != null) handler(this, e);
        }

        public void LoadFavoris()
        {
            if (!IsolatedStorageSettings.ApplicationSettings.Contains("favoris"))
                IsolatedStorageSettings.ApplicationSettings["favoris"] = new ObservableCollection<Station>();

            /// A chaque chargement de l'application, nous récupérons de nouveaux objets stations,
            /// même si les IDs correspondent, nous avons deux objets différents. Il faut donc reconstruire la
            /// liste des favoris
            var sf = (ObservableCollection<Station>) IsolatedStorageSettings.ApplicationSettings["favoris"];

            List<Station> temp = (from s in Stations
                                  where sf.Contains(s, new LambdaComparer<Station>((x, y) => x.Id == y.Id))
                                  select s).ToList();
            temp.ForEach(t =>
                             {
                                 StationsFavorites.Add(t);
                                 t.StationFavorite = true;
                             });
            Debug.WriteLine("coll");
        }

        public void SaveFavoris()
        {
            IsolatedStorageSettings.ApplicationSettings["favoris"] = StationsFavorites;
            IsolatedStorageSettings.ApplicationSettings.Save();
        }
    }
}